% HAM-ISI feedback example - controls lecture II, part 2
% Makes a HAM-ISI model along a horizontal degree of freedom (X or Y). The
% model is created in the state space format. Then a simple feedback control filter
% is created. 
% 
% 6 plots are generated:
% 1) plant TF bode plot for actuator force to displacement  (m/N)
% 2) plant TF bode plot for ground displacement to displacement  (m/m)
% 3) Bode plot of the loop gain, after the controller is designed
% 4) Nyquist plot of the loop gain, as another check on stability
% 5) Nichols plot of the loop gain, as yet another check on stability
% 6) Step response of the closed loop transfer function from ground to
% plant displacement.
% 
% Brett Shapiro
% Last updated
% 28 March 2016

% clear all workspace variables, close all figures, and clear the command line
clear
close all
clc

%% Plot settings

% universal settings
textsize = 30;

% bode plot options
bodeopts = bodeoptions;
bodeopts.Grid = 'On';
bodeopts.FreqUnits = 'Hz';
bodeopts.MagUnits = 'abs';
bodeopts.MagScale = 'log';
bodeopts.Title.FontSize = textsize;
bodeopts.YLabel.FontSize = textsize;
bodeopts.XLabel.FontSize = textsize;
bodeopts.TickLabel.FontSize = 25;

% nyquist plot options
nyquistopts = nyquistoptions;
nyquistopts.ShowFullContour = 'off';
nyquistopts.FreqUnits = 'Hz';
nyquistopts.Title.FontSize = textsize;
nyquistopts.YLabel.FontSize = textsize;
nyquistopts.XLabel.FontSize = textsize;
nyquistopts.TickLabel.FontSize = 25;

% nichols plot options
nicholsopts = nicholsoptions;
nicholsopts.Grid = 'off';
nicholsopts.FreqUnits = 'Hz';
nicholsopts.Title.FontSize = textsize;
nicholsopts.YLabel.FontSize = textsize;
nicholsopts.XLabel.FontSize = textsize;
nicholsopts.TickLabel.FontSize = 25;

%% HAM-ISI horizontal parameters, from G070156 slide 65
damp_ratio = 0.1; % damping ratio: 0 -> no damping, 1 -> critical damping; note the quality factor = 1/(2*damp_ratio).
g=9.81; % gravitational acceleration in m/s^2
m=1900; % mass of stage in kg
fl = 0.1482; % flexure length in m
Kxx = m*g/fl; % horizontal stiffness of stage
fx = sqrt(Kxx/m)/(2*pi); % Undamped horizontal natural frequency in Hz
Cxx = 2*damp_ratio*(2*pi*fx)*m; % viscous damping constant in N/(m/s)

%% State space model
A = [   0       1
     -Kxx/m  -Cxx/m];
B = [ 0   0
     1/m Kxx/m]; % column 1: force inputl column 2: ground input. Note the ground input does not go through the damper. Doing so requires changing the velocity state to something else. See T1400023.
C = [1 0]; % true displacement output
D = [0  0];
HAMmodel = ss(A,B,C,D); % state space model

% Plant transfer functions
Pf = HAMmodel(1,1); % force to displacement plant
Pg = HAMmodel(1,2); % ground displacement to displacement plant

% make the force to displacement bode plot of the plant transfer function; magnitude units are m/N
figure
bodeplot(Pf,bodeopts)
title('HAM-ISI Example Bode Plot, force to displacement transfer function')

% make the ground displacement to plant displacement bode plot; magnitude units are m/m
figure
bodeplot(Pg,bodeopts);
title('HAM-ISI Example Bode Plot, ground to displacement transfer function')

%% Simple control filter

% filter
ugf = 25; %Hz, unity gain frequency
C = zpk(-2*pi*[2;ugf/3],-2*pi*[0.01;ugf*3;100],1);

% scale C so the loop gain magnitude is 1 at the ugf
C = C/abs(freqresp(Pf*C,2*pi*ugf));

% Loop gain
L = Pf*C;

% Closed loop TF from ground displacement to plant displacement
CL = Pg / (1 + L);

% Bode plot of the loop gain TF L
figure
bodeplot(L,bodeopts);
% hold on
% bodeplot(C);
% bodeplot(Pf);
title('Loop gain transfer function')

% Nichols plot of the loop gain TF L - to examine stability; should not circle the -1 point
figure
nyquistplot(L,nyquistopts)

% Nyquist plot of the loop gain TF L - another way to examine stability; should not envelop the -1 point
figure
nicholsplot(L,nicholsopts)

% Step response from ground to plant displacement. If the low frequency
% loop gain is large, the final value will be close to zero, as it should
% be.
figure
step(CL,0:0.001:10),grid on
title('Closed loop step response from ground displacement to plant displacement')
